local EventManager =  FlexBar2:NewModule("EventManager");

local EventTable = {};

EventManager.ButtonMixin = FlexBar2.OO.Mixin
{
	"RegisterEvent", -- Custom RegisterEvent function, Registers the event with AceEvent if it ain't already. It is table-based, read its code for details ;)
	"UnregisterEvent",
	"RegisterScript", -- Custom version of the SetScript function. Very similar to my custom RegisterEvent function except it is for SetScript stuff :)
	"UnregisterScript",
	"UnregisterAll",
};

function EventManager.ButtonMixin:RegisterEvent(Event, MethodName) -- Note: Events are not shared with the groupmembers as they already get them by registering the event anyway
	FlexBar2:Debug("Registering " , Event, " with " , MethodName);
	if((type(MethodName) == "string" and type(self[MethodName]) == "function") or type(MethodName) == "function") then
		if(not FlexBar2:IsEventRegistered(Event)) then
			FlexBar2[Event] = function(...)
				for Button , EventTable in pairs(EventTable[Event]) do
					for _, MethodName in pairs(EventTable) do
						if(type(MethodName) == "string") then
							Button[MethodName](Button, Event, ...);
						elseif(type(MethodName) == "function") then
							MethodName(Button, Event, ...);
						end
					end
				end
			end
			FlexBar2:RegisterEvent(Event);
		end
		if(type(EventTable[Event]) ~= "table") then
			EventTable[Event] = {};
		end
		if(type(EventTable[Event][self]) ~= "table") then
			EventTable[Event][self] = {};
		end
		table.insert(EventTable[Event][self], MethodName);
	else
		return false;
	end
end

function EventManager.ButtonMixin:UnregisterEvent(Event, MethodName)
	if(MethodName) then
		return FlexBar2:RemoveByValue(EventTable[Event][self], MethodName);
	else
		for n,v in pairs(EventTable[Event][self]) do
			table.remove(EventTable[Event][self], n);
		end
	end
end


function EventManager.ButtonMixin:RegisterScript(Trigger, MethodName) -- Widget scripts are sent to the group handler if there is one available, if not they are not shared
    FlexBar2:Debug("Registering ", Trigger, " ," , MethodName);
    if((type(MethodName) == "string" and type(self[MethodName]) == "function") or type(MethodName) == "function") then
		if(type(self.ScriptTable) ~= "table") then
			self.ScriptTable = {};
		end
		if(type(self.Frame:GetScript(Trigger)) ~= "function") then
			self.Frame:SetScript(Trigger, function(...)
				for _ , ScriptName in ipairs(self.ScriptTable[Trigger]) do
					if(type(ScriptName) == "string") then
						if(Trigger ~= "OnUpdate") then FlexBar2:Debug("Triggering ", ScriptName, " For Handler: ", Trigger); end
						self[ScriptName](self, Trigger, ...);
					elseif(type(ScriptName) == "function") then
						ScriptName(self, Trigger, ...);
					end
				end
			end);
		end
		if(type(self.ScriptTable[Trigger]) ~= "table") then
			self.ScriptTable[Trigger] = {};
		end
		table.insert(self.ScriptTable[Trigger], MethodName);
	else
		return false;
	end
end

function EventManager.ButtonMixin:UnregisterScript(Trigger, MethodName)
	if(type(self.ScriptTable) ~= "table") then
		self.ScriptTable = {};
	end
	if(MethodName) then
		return FlexBar2:RemoveByValue(self.ScriptTable[Trigger], MethodName);
	else
		for n,v in pairs(self.ScriptTable[Trigger]) do
			table.remove(self.ScriptTable[Trigger], n);
		end
	end
end
